/*
 lksteh_wqcounter -- wait queue counting handler
  
 Copyright (C) HITACHI,LTD. 2004
 WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
 Created by M.Hiramatsu <hiramatu@sdl.hitachi.co.jp>
  
 The development of this program is partly supported by IPA
 (Information-Technology Promotion Agency, Japan).
  
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

#include <linux/version.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/lkst_private.h>
#include <linux/wait.h>
#include "../include/extra_etypes.h"

MODULE_AUTHOR("M.Hiramatsu <hiramatu@sdl.hitachi.co.jp>");
MODULE_DESCRIPTION("");
MODULE_LICENSE("GPL");

/* Declaration of hook header */
LKST_ETYPE_DEF_MODULE(EID_PROCESS_CNT_WQ, INLINE, PROCESS_CNT_WQ, "waitqueue counting", \
		      "pointer to wait_queue_head", \
		      "pointer to added process", \
		      "pointer to call address",\
		      "waitqueue length");

static void lkst_evhandler_wqcounter(void *phookrec, int event_type,
				     lkst_arg_t arg1, lkst_arg_t arg2,
				     lkst_arg_t arg3, lkst_arg_t arg4)
{
	preempt_disable();	/*IMPORTANT*/
	if (event_type == LKST_ETYPE_PROCESS_ADD_WQ) {
		int i=0;
		struct list_head *p, *h;
		if (arg1) {
			h = &(((wait_queue_head_t*)((long)arg1))->task_list);
			list_for_each(p,h) i++;
		}
		lkst_evhandlerprim_entry_log(LKST_ETYPE_PROCESS_CNT_WQ,
					     arg1, arg2, arg3, LKST_ARG(i));
	}
	preempt_enable();	/*VERY IMPORTANT*/
}


static int mod_init(void);
static void mod_cleanup(void);
module_init(mod_init);		/*define as initializer*/
module_exit(mod_cleanup);	/*define as terminator*/

static LKST_EH_DEV_DEF(wqcounter,
		       lkst_evhandler_wqcounter,
		       NULL,NULL,NULL);

static int mod_init(void)
{
	int	retval;
	/* Initialization of hook header */
	retval = lkst_hook_etype_register(&LKST_ETYPE_INFO(PROCESS_CNT_WQ));
	if (retval < 0) {
		printk("lksteh_wqcounter: Error: failed to etype register (%d)\n", 
		       retval);
		goto init_err;
	}
	/* register an event handler */
	retval = lkst_eh_device_register(&LKST_EH_DEV(wqcounter));
	if (retval < 0) {
		printk("lksteh_wqcounter: Error: failed to evhandler register (%d)\n", 
		       retval);
		goto init_err;
	}
	return 0;
	
	init_err:
	mod_cleanup(); /*If occurs an error, invokes terminator.*/
	return retval;
}


static void mod_cleanup(void)
{
	if (LKST_EH_DEV(wqcounter).id != LKST_EVHANDLER_ID_VOID) {
		lkst_eh_device_unregister(&LKST_EH_DEV(wqcounter));
	}
	lkst_hook_etype_unregister(&LKST_ETYPE_INFO(PROCESS_CNT_WQ));
}
