/*
 network sending delay time distribution analyer

 Copyright (C) HITACHI,LTD. 2004-2005
 WRITTEN BY HITACHI SYSTEMS DEVELOPMENT LABORATORY,
 Created by M.Hiramatsu <hiramatu@sdl.hitachi.co.jp>
  
 The development of this program is partly supported by IPA
 (Information-Technology Promotion Agency, Japan).
  
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */

#include <stdio.h>
#include <errno.h>
#include <linux/lkst.h>
#include <linux/lkst_buffer.h>
#include <lkstla.h>
#include <string.h>

static slot_hkey info_key_netsend(struct lkst_log_record *rec_start, struct lkst_log_record *rec_end);
static slot_hkey session_key_netsend(struct lkst_log_record *rec);
static char * get_alias_netsend(slot_hkey key);
static ga_type_t get_type_netsend(struct lkst_log_record *rec);
static int init_netsend(void);

struct gate_analyzer netsend_analyzer = {
	.name = "netsend",
	.description = "packet send delay time analyzer",
	.cols = {
		[COL_KEY] = { "device", ADDR_LEN, "%0*lx" },
		[COL_ALIAS] = { "device", ADDR_LEN + 2, "%*s" },
		[COL_METRIC] = { "processing-time" },
	},

	GA_INIT_SESSION_ANALYZER
	GA_INIT_1TON_SESSION
	GA_INIT_RET_SESSION_TIME
	GA_INIT_NO_EXTRAOPTS
	GA_INIT_RET_EVENT_PID
		
	.info_key = info_key_netsend,
	.session_key = session_key_netsend,
	.get_alias = get_alias_netsend,

	.get_type = get_type_netsend,

	.init = init_netsend,
};


static int init_netsend(void)
{
	return 0;
}

static slot_hkey info_key_netsend(struct lkst_log_record *start,
				  struct lkst_log_record *end)
{
	return ARG2UL(end);
}
static slot_hkey session_key_netsend(struct lkst_log_record *rec)
{
	return ARG1UL(rec);
}

static char buf[256];
static char * get_alias_netsend(slot_hkey key)
{
	sprintf(buf,"0x%0*lx", ADDR_LEN, key);
	return buf;
}

static ga_type_t get_type_netsend(struct lkst_log_record *rec)
{
	switch(rec->posix.log_event_type) {
	case LKST_ETYPE_NET_PKTSEND:
		return GAT_START;
	case LKST_ETYPE_NET_START_XMIT:
		if (ARG3UL(rec) == 0) return GAT_END;
	default:
		return GAT_IGNORE;
	}
}
