#!/bin/sh
# LKST release package script
# COPYRIGHT (C) HITACHI,LTD. 2003-2004,2005
# Written by M.Hiramatsu <hiramatu@sdl.hitachi.co.jp>
# Usage: scripts/release [clean]
# !WARNIG!: you should check following 2 points before using this script.
# 1. Make sure that /usr/src/linux points the kernel patched release-version patches of LKST.
# 2. Make sure that you can build rpm packages.

SCRDIR=`dirname $0`
[ "$SCRDIR" ] || SCRDIR=`pwd`
source $SCRDIR/../lkst.version
source $SCRDIR/functions

SCRDIR=`rp2ap $SCRDIR`
ROOTDIR=`rp2ap $SCRDIR/../`
PATCHDIR=$ROOTDIR/patches

RELDIR=$ROOTDIR/Releases
LOGFILE=$RELDIR/release.log

if [ "$1" = clean ] ; then
	echo -n "cleaning up ..."
	make distclean &> /dev/null
	rm -rf $RELDIR
	echo "done"
	exit 0
fi

[ -e $RELDIR ] && echo "$RELDIR already exists, abort." && exit 1
mkdir $RELDIR

#echo -n "compressing patches ..."
#( cp $PATCHDIR/*.patch $RELDIR ;
#  gzip -f $RELDIR/*.patch 
#  ) &> $LOGFILE
#if [ $? -ne 0 ]; then echo "failed" ; exit 1; fi
#mv $RELDIR/linux26.patch.gz $RELDIR/lkst-$VERSION.linux26.patch.gz
#echo "done"

echo -n "converting documents ..."
( nkf --windows $ROOTDIR/Install > $RELDIR/Install-$VERSION.txt
  nkf --windows $ROOTDIR/howto.txt > $RELDIR/Howto-$VERSION.txt
  nkf --windows $ROOTDIR/changelog > $RELDIR/Changelog-$VERSION.txt
  ) >> $LOGFILE 2>&1
if [ $? -ne 0 ]; then echo "failed" ; exit 1; fi
echo "done"

echo -n "configuring packages ..."
( cd $ROOTDIR
  make config 
  ) >> $LOGFILE  2>&1
if [ $? -ne 0 ]; then echo "failed" ; exit 1; fi
echo "done"

echo -n "packaging lkstutils ..."
( cd $ROOTDIR/lkstutils
  make dist
  mv lkstutils-*.tar.gz $RELDIR/
  ) >> $LOGFILE 2>&1
if [ $? -ne 0 ]; then echo "failed" ; exit 1; fi
echo "done"

echo -n "packaging lkstaddons ..."
( cd $ROOTDIR/lkstaddons
  make dist
  mv lkstaddons-*.tar.gz $RELDIR/
  ) >> $LOGFILE 2>&1
if [ $? -ne 0 ]; then echo "failed" ; exit 1; fi
echo "done"

echo -n "packaging lkstlogtools ..."
( cd $ROOTDIR/lkstlogtools
  make dist
  mv lkstlogtools-*.tar.gz $RELDIR/
  ) >> $LOGFILE 2>&1
if [ $? -ne 0 ]; then echo "failed" ; exit 1; fi
echo "done"

echo -n "building the rpm package of lkstutils ..."
( export LANG=C
  echo See $RELDIR/rpm.log
  rpmbuild -ta $RELDIR/lkstutils-*.tar.gz &> $RELDIR/rpm.log
  [ $? -eq 0 ] && mv `grep ^Wrote $RELDIR/rpm.log | cut -d" " -f2 ` $RELDIR
  ) >> $LOGFILE 2>&1
if [ $? -ne 0 ]; then echo "failed" ; exit 1; fi
echo "done"

echo -n "building the rpm package of lkstlogtools ..."
( export LANG=C
  echo See $RELDIR/rpm2.log
  rpmbuild -ta $RELDIR/lkstlogtools-*.tar.gz &> $RELDIR/rpm2.log
  [ $? -eq 0 ] && mv `grep ^Wrote $RELDIR/rpm2.log | cut -d" " -f2 ` $RELDIR
  ) >> $LOGFILE 2>&1
if [ $? -ne 0 ]; then echo "failed" ; exit 1; fi
echo "done"

echo -n "packaging lkst ..."
( cd $ROOTDIR
  make dist
  mv lkst-*.tar.gz $RELDIR/
  ) >> $LOGFILE 2>&1
if [ $? -ne 0 ]; then echo "failed" ; exit 1; fi
echo "done"

echo -n "making checksums ..."
( cd $RELDIR
  for i in *.tar.gz ;do 
  md5sum $i > $i.md5
  done
)
echo "done"
